#!/usr/bin/python
################################################################################
#	BdsCloneArray.py	BDS API example code for a BDS MetaData Client
#				T.Barnaby,	BEAM Ltd,	2013-02-04
################################################################################
#
# This example shows how to clone an Array of channels.
# The example assumes the BDS test data is installed.
#
# By default it clones the Station TSB01 with all MetaData from
# 2008-01-01T00:00:00 to 2008-01-01T00:00:00. This means just one
# item of Metadata is cloned and the time period of this is set to all time.
# All Metadata is cloded, including the Sensors and Digitisers.
#
# The operation is carried out in a BDS transaction. So any failure will
# mean that no information will be added to the BDS database.
#
import sys
import getopt
from bdslib import *

# Program configuration information (Command line arguments etc)
class Config:
	def __init__(s):
		s.hostName = "localhost";
		s.user = "testAdmin";
		s.password = "beam00";
		s.stationFrom = "TSB01";
		s.stationTo = "TSC01";
		s.startTime = "2008-01-01T00:00:00.000000";

# Function to get Response for a channel
def bdsCloneStation(bds, config):
	err		= BError();
	selection	= Selection();

	# Start a transaction
	bds.transactionStart();
		
	# Set up selection
	selection.startTime.setString(config.startTime);
	selection.endTime.setString(config.startTime);
	selection.channels.append(SelectionChannel("TT", config.stationTo, "", ""));
#	bdsDumpSelection(selection);

	# First clone the station, if needed
	(err, stations) = bds.stationGetList(selection);
	if(err):
		return err;
		
	if(stations.number() == 0):
		station		= Station();

		print "Add station:", config.stationTo;
		station.name = config.stationTo;
		station.type = "station";

		(err, station.id) = bds.stationUpdate(1, station);
		if(err):
			return err;

	# Get list of channels that match
	selection.channels.clear();
	selection.channels.append(SelectionChannel("TT", config.stationFrom, "", ""));
	(err, channels) = bds.channelGetList(selection);
	if(err):
		return err.set(1, "Error: Getting channels: " + err.getString());

	# Prints and mods this response
	for channel in channels:
		print "Clone channel:", channel.channel;
		channel.id = 0;
		channel.station = config.stationTo;
		channel.description += " (Cloned)";
		(err, channel.id) = bds.channelUpdate(1, channel);
		if(err):
			return err;
		
		selection.channels.clear();
		selection.channels.append(SelectionChannel("TT", config.stationFrom, channel.channel, ""));

		# Clone Calibrations
		(err, calibrations) = bds.calibrationGetList(selection);
		if(err):
			return err.set(1, "Error: Getting calibrations: " + err.getString());

		for calibration in calibrations:
			calibration.id = 0;
			calibration.startTime.setFirst();
			calibration.endTime.setLast();
			calibration.station = config.stationTo;
			(err, calibration.id) = bds.calibrationUpdate(1, calibration);
			if(err):
				return err;
		
		# Clone Responses
		(err, responses) = bds.responseGetList(selection);
		if(err):
			return err.set(1, "Error: Getting responses: " + err.getString());
		for response in responses:
			response.id = 0;
			response.startTime.setFirst();
			response.endTime.setLast();
			response.station = config.stationTo;
			(err, response.id) = bds.responseUpdate(1, response);
			if(err):
				return err;
		
		# Clone Instruments
		(err, instruments) = bds.channelInstrumentGetList(selection);
		if(err):
			return err.set(1, "Error: Getting instruments: " + err.getString());
		for instrument in instruments:
			# Clone Sensor
			(err, sensor) = bds.sensorGet(instrument.sensorId);
			if(err):
				return err;

			sensor.id = 0;
			sensor.name += " (Cloned)";
			(err, sensor.id) = bds.sensorUpdate(1, sensor);
			if(err):
				return err;
			
			# Clone Digitiser
			(err, digitiser) = bds.digitiserGet(instrument.digitiserId);
			if(err):
				return err;

			digitiser.id = 0;
			digitiser.name += " (Cloned)";
			(err, digitiser.id) = bds.digitiserUpdate(1, digitiser);
			if(err):
				return err;
			
			instrument.id = 0;
			instrument.startTime.setFirst();
			instrument.endTime.setLast();
			instrument.channelId = channel.id;
			instrument.sensorId = sensor.id;
			instrument.digitiserId = digitiser.id;
			(err, instrument.id) = bds.channelInstrumentUpdate(1, instrument);
			if(err):
				return err;

	# End transaction with Ok status
	bds.transactionEnd(0);

	return err;

def usage():
	print "Usage: bdsCloneStation.py [options]";
	print "	-h,--help              - Help";
	print "--host <host>           - BDS host name (default localhost)";
	print "--user <user:password>  - BDS user and password (default testAdmin:beam00)";


def main():
	# Setup default configuration
	config = Config();
	
	# Parse commane line arguments
	try:
		opts, args = getopt.getopt(sys.argv[1:], "h", ["help", "host=", "user="]);

	except getopt.GetoptError:
        	# print help information and exit:
        	usage();
        	sys.exit(1);

	for o, a in opts:
		if(o in ("-h", "--help")):
			usage();
			sys.exit(1);
		elif(o == "--host"):
			config.hostName = a;
		elif(o == "--user"):
			config.user = a.split(":")[0];
			config.password = a.split(":")[1];
		else:
			usage();
			sys.exit(1);

	# Create DataAccess object to connect to BDS Server
	bds = AdminAccess();

	# Connect to the DataAccess service
	err = bds.connectService("//" + config.hostName + "/bdsAdminAccess");
	if(err):
		print "Error: " + str(err);
		return 1;

	# Connect to service with user information
	err = bds.connect(config.user, config.password);
	if(err):
		print "Error: " + str(err);
		return 1;

	err = bdsCloneStation(bds, config);
	if(err):
		print "Error: " + str(err);
		return 1;

if __name__ == "__main__":
	main();
