/*******************************************************************************
 *	BdsDataFile.h	Data File Access
 *			T.Barnaby,	BEAM Ltd,	2008-06-24
 *******************************************************************************
 */
#ifndef BdsDataFile_H
#define	BdsDataFile_H

#include <BString.h>
#include <BFile.h>
#include <BdsLib.h>
#include <BdsD.h>

namespace Bds {

class DataBlockPos {
public:
				DataBlockPos(BTimeStamp startTime = 0, BTimeStamp endTime = 0, BUInt64 position = 0, BUInt order = 0, int ref = 0, BUInt numSamples = 0) : startTime(startTime), endTime(endTime), position(position), order(order), ref(ref), numSamples(numSamples){}
	int			operator<(const DataBlockPos& b) const { return startTime < b.startTime;	}
	BTimeStamp		startTime;
	BTimeStamp		endTime;
	BUInt64			position;
	BUInt			order;
	int			ref;
	BUInt			numSamples;
};

class DataFileOptions {
public:
				DataFileOptions(int options = 0){	ooptionList = options; }
				operator int (){	return ooptionList;	}
	DataFileOptions&	operator|=(int o){	ooptionList |= o; return *this; }
			
	int		ooptionList;
	BArray<BUInt>	oignoreBlockList;
};

/// This class defines the interface for generic data file access
class DataFile {
public:
	enum DataOrder		{ DataOrderUnknown, DataOrderAll, DataOrderSample, DataOrderChannel };
	enum Features		{ FeatureNone = 0x00, FeatureCanWrite = 0x01, FeatureCanRead = 0x02 };
	enum WriteOptionsList	{ WriteOptionNone = 0x00, WriteOptionSensorData = 0x01, WriteOptionNoMetadata = 0x02 };
	enum ReadOptionsList	{ ReadOptionNone = 0x00, ReadOptionValidate = 0x01, ReadOptionFileNameProcess = 0x02, ReadOptionValidateCorruptions = 0x04, ReadOptionReorder = 0x08, ReadOptionDeleteDuplicates = 0x10, ReadOptionInfoExtra = 0x20, ReadOptionIgnoreSamplerate = 0x40, ReadOptionPrintBlocks = 0x80 };

				DataFile();
	virtual			~DataFile();
	
	virtual void		init();								///< Initialise
	virtual BError		open(BString fileName, BString mode);				///< Open the file for read or write
	virtual BError		close();							///< Close the file

	virtual BError		setFormat(BString format);					///< Set the sub-format
	virtual	BString		getFileName();							///< Return the file name
	virtual DataOrder	getDataOrder();							///< Get the expected order of writing data, by sample or by channel
	virtual int		getFeatures();							///< Get bitmask of supported features
	
				// Write routines
	virtual BError		setInfo(DataInfo& dataInfo, ChannelInfos& channelInfos, WriteOptionsList options = WriteOptionNone);		///< Set information on data for write
	virtual	BError		start(BUInt channel, BUInt segment);					///< Start next segment of data
	virtual BError		writeData(DataBlock& data);						///< Write a block of data
	virtual BError		end();									///< End segment

				// Read routines
	virtual BError		fileNameProcess();							///< Parse the file name for a date/time
	virtual BError		getFormat(BString& format);						///< Get sub-format
	virtual BError		getInfo(DataInfo& dataInfo, DataFileOptions options, BList<DataError>& errors);	///< Get info on data
	virtual BError		seekBlock(BUInt32 channel, BUInt segment, BTimeStamp time, BUInt32& blockNumber, BUInt64& sampleNumber, DataBlock& data);	///< Find requested block on given channel given a time
	virtual BError		readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& dataBlock);	///< Read a block

	virtual BError		getMetaData(ChannelInfos& channelInfos);				///< Return all known MetaData in the file
	
				// Misc functions
	void			dataErrorFixup(DataInfo& dataInfo, BList<DataError>& errors);		///< Fixup data errors, mainly start/end times to be within data
	BInt64			timeCompare(BTimeStamp t1, BTimeStamp t2, BUInt diff);			///< Compare timestamps with a margin
	int			duplicateCheck(DataBlock& data1, DataBlock& data2, BUInt channel = 0);	///< Check if blocks are duplicates
	BUInt64			getFilePosition();
	static DataFormat	getFormats();							///< Get list of supported formats

protected:
	BString			ofileName;
	BString			omode;
	BTimeStamp		ofileNameTime;
	BFile			ofile;
	BString			oformat;
};

}
#endif
