/*******************************************************************************
 *	BdsDataFileBds.h	Bds internal data file format
 *				T.Barnaby,	BEAM Ltd,	2008-05-06
 *******************************************************************************
 *
 *	Supported sub-formats:
 *		BDS, BDS-SM	BDS Sample Multiplexed
 *		BDS-CM		BDS Channel Multiplexed
 *		BDS-SM-CC	BDS Sample Multiplexed, Canadian Compression
 */
#ifndef BdsDataFileBds_H
#define BdsDataFileBds_H	1

#include <BdsDataFile.h>
#include <BBuffer.h>

namespace Bds {

enum BdsDataType		{ BdsDataTypeBlock = 0x42534442, BdsDataTypeInfo = 0x30534442, BdsDataTypeData = 0x31534442, BdsDataTypeInfoExtra = 0x32534442 };

// Fixed size BDS Data Block
struct BdsDataBlockHeader {
	BUInt32		type;			///< Blocks type, contains magic number for synchronisation
	BUInt32		length;			///< Packet length
	BUInt32		packetOffset;		///< Offset to first packet header within block or 0 if no packet header within block
};

struct BdsDataBlock {
	BdsDataBlockHeader	header;
	char			data[];		///< The packet data
};

struct BdsDataPacketHeader {
	BUInt32		type;			///< Packets type
	BUInt32		length;			///< Length in bytes of packet
	BUInt32		streamlet;		///< The streamlet id
	BUInt32		sequence;		///< The streamlet packet sequence number
	BUInt32		checksum;		///< Checksum of packet
	BTimeStamp	startTime;		///< The time of the first sample
	BTimeStamp	endTime;		///< The time of the last sample + 1
};

// BdsDataPacket
class BdsDataPacket : public BBufferStore {
public:
			BdsDataPacket();
			~BdsDataPacket();

	void		clear();
	void		reset();

	void		setChecksumAndLength();
	BError		validateChecksum();

	BError		setHeader(BdsDataPacketHeader& header);
	BError		getHeader(BdsDataPacketHeader& header);
	
	void		dump();
};

class BdsDataStreamlet {
public:
			BdsDataStreamlet() : packetNumber(0), position(0){}

	BUInt32		packetNumber;
	BUInt64		position;
};

class BdsDataSegment {
public:
			BdsDataSegment() : streamlet(0), channel(0), numChannels(0), numBlocks(0){};

	BUInt32		streamlet;
	BTimeStamp	startTime;
	BTimeStamp	endTime;
	BUInt32		channel;
	BUInt32		numChannels;
	BUInt32		numBlocks;
	BUInt32		numSamples;
	double		sampleRate;
};

class BdsDataBlockPos {
public:
				BdsDataBlockPos(BTimeStamp startTime = 0, BTimeStamp endTime = 0, BUInt32 channel = 0, BUInt32 numChannels = 0, BUInt32 segment = 0, BUInt64 position = 0, BUInt64 numSamples = 0) :
					startTime(startTime), endTime(endTime), channel(channel), numChannels(numChannels), segment(segment), position(position), numSamples(numSamples){}
	int			operator<(const DataBlockPos& b) const { return startTime < b.startTime; }

	BTimeStamp		startTime;
	BTimeStamp		endTime;
	BUInt32			channel;
	BUInt32			numChannels;
	BUInt32			segment;
	BUInt64			position;
	BUInt64			numSamples;
};

/// This class implements the BDS Data File/Stream access system
class DataFileBds : public DataFile {
public:
	enum			{ StreamsMax = 256 };
	enum			{ DefaultBlockSize = 65536 };
	enum PackFormat		{ PackFormat_Unknown = 0, PackFormat_SM = 1, PackFormat_CM = 2, PackFormat_SM_CC = 3 };
	
	// Generic access
				DataFileBds();
				~DataFileBds();
	
	BError			open(BString fileName, BString mode);							///< Open the file for reading or writing
	BError			close();										///< Close file

				// Write routines
	BError			setFormat(BString format);								///< Sets the sub-format
	BError			setInfo(DataInfo& dataInfo, ChannelInfos& channelInfos, WriteOptionsList options = WriteOptionSensorData);		///< Sets the information
	BError			writeData(DataBlock& data);								///< Writes a data block to the file

				// Read routines
	DataOrder		getDataOrder();
	BError			getInfo(DataInfo& dataInfo, DataFileOptions options, BList<DataError>& errors);			///< Get information on open file
	BError			seekBlock(BUInt32 channel, BUInt segment, BTimeStamp time, BUInt32& blockNumber, BUInt64& sampleNumber, DataBlock& dataBlock);	///< Find the block that contains the samples for the time requested
	BError			readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& dataBlock);		///< Read the data block for the given channel or all channels if blockNumber is 0

	static DataFormat	getFormats();					///< Get the names of the supported formats

	BError			setDiskBlockSize(BUInt32 blockSize);				///< Sets up file/stream block size
	BUInt32			getDiskBlockSize();						///< Returns the data block size in bytes

private:
	void			clear();

	// Low level file access
	BError			setInfoRepeat(BUInt32 repeat);								///< Sets the reapeat period for the information packets
	BError			infoSet(BUInt32 streamlet, BTimeStamp startTime, BTimeStamp endTime, BDictString& info);	///< Sets the information packet and writes this to the file/stream
	BError			infoGet(BdsDataPacket& packet, BDictString& info);					///< Gets the information from an information packet
	BError			addInfoPacket();
	BError			addInfoExtraPacket();
	BError			streamletFromChannel(BUInt channel, BUInt& streamlet);					///< Find streamlet given channel

	BError			dataAppend(DataBlock& data);								///< Appends a data block to the file
	BError			dataGet(BUInt channel, BdsDataPacket& packet, DataBlock& data);				///< Gets the data from a data packet

	// Low Level Packet I/O
	BError			packetRead(BdsDataPacket& packet);							///< Reads a packet from the file
	BError			packetWrite(BdsDataPacket& packet);							///< Writes a packet to the file
	
	// Low level Disk Block access functions
	void			diskBlockReset();						///< Resets to disk block 0
	BError			diskBlockWrite(void* data, BUInt32 numBytes, int header = 0);	///< Writes data to a block in the file
	BError			diskBlockWriteFlush();						///< Flushes a semi-filled block to the file
	BError			diskBlockRead(void* data, BUInt32 numBytes, int header = 0);	///< Reads data from a block
	BError			diskBlockSeek(BUInt64 position);

	BString			oformat;				///< The format	
	PackFormat		opackFormat;				///< The data packet format
	DataInfo		odataInfo;				///< Information on the channel data
	ChannelInfos		ochannelInfos;				///< Channel meta data information
	
	BUInt32			oinfoRepeat;				///< How often to output info packet
	BDictString		oinfo;					///< Info packet output
	BdsDataPacketHeader	oinfoHeader;				///< Info packet header

	// Disk Block access data
	BUInt32			odiskBlockSize;				///< The block size
	BdsDataBlock*		odiskBlock;				///< Data block buffer
	BUInt64			odiskPosition;				///< The disk block position

	BArray<BdsDataStreamlet>	ostreamlets;			///< Streamlet sequence numbers and positions
	BdsDataPacket			opacket;			///< Temporary packet for header/data packet generation
	
	BArray<BdsDataSegment>	osegments;				///< Data channel segments
	BArray<BArray<BdsDataBlockPos> >	oblockPositions;	///< Data block positions for seeking
};
}
#endif
